import z from 'zod';
import OpenAI from 'openai';
import * as zod_to_json_schema from 'zod-to-json-schema';

type ValuesOf<T> = T[keyof T];

/**
 * The type of authentication.
 */
declare const AuthType: {
    readonly NONE: "None";
    readonly TOKEN: "Token";
    readonly OAUTH: "OAuth";
};
type TAuthType = ValuesOf<typeof AuthType>;
/**
 * Base type for configuring authentication.
 */
type TAuthConfigBase = {
    /**
     * A configuration object for the button users will see before authenticating.
     */
    button: {
        /**
         * The text to display on the button.
         *
         * @example
         * The following text is used for the `Google Calendar` authentication button.
         * ```
         * "Continue with Google"
         * ```
         *
         * @example
         * The following text is used for the `Linear` authentication button.
         * ```
         * "Continue with Linear"
         * ```
         */
        text: string;
    };
    /**
     * The URL to the privacy policy for the authentication.
     *
     * @example
     * The following URL is used for the `Google People API` authentication.
     * ```
     * "https://developers.google.com/people/v1/getting-started"
     * ```
     */
    policyReferenceURL?: string;
    /**
     * A human readable name for the scope of the authentication.
     *
     * @example
     * The following name is used for the `Google Calendar` read scope authentication.
     * ```
     * "Google Calendar"
     * ```
     *
     * @example
     * The following name is used for the `Google Contacts` read scope authentication.
     * ```
     * "Google Contacts"
     * ```
     */
    humanReadableName: string;
    /**
     * A human readable description for the scope of the authentication.
     *
     * @example
     * The following description is used for the `Google Calendar` read scope authentication.
     * ```
     * "Read-only access to your Google Calendar"
     * ```
     *
     * @example
     * The following description is used for the `Google Calendar` write scope authentication.
     * ```
     * "Read and write access to your Google Calendar"
     * ```
     */
    humanReadableDescription: string;
};
/**
 * Configuration for OAuth-based authentication.
 */
type TOAuthConfig = TAuthConfigBase & {
    /**
     * An non-empty array of scopes required for the OAuth authentication.
     *
     * @example
     * The following scopes are used for the `Google Contacts` authentication.
     * ```
     * ["https://www.googleapis.com/auth/contacts.other.readonly", "https://www.googleapis.com/auth/contacts.readonly"]
     * ```
     */
    scopes: [string, ...string[]];
    /**
     * A URL that will take the user to where they can generate their own OAuth app to obtain a client ID and client secret.
     */
    oauthAppGenerationURL: string;
    /**
     * A URL to documentation that helps users understand the OAuth process or set it up.
     */
    documentationURL?: string;
} & ({
    /**
     * The URL of the OAuth 2.0 Discovery Endpoint.
     *
     * @example
     * For google, this is `https://accounts.google.com/.well-known/openid-configuration`.
     */
    discoveryEndpoint: string;
} | {
    discoveryEndpoint: undefined;
    /**
     * The URL of the authorization endpoint for OAuth.
     *
     * @example
     * For google, this is `https://accounts.google.com/o/oauth2/v2/auth`.
     */
    authorizationEndpoint: string;
    /**
     * The URL of the token endpoint for obtaining access and refresh tokens.
     *
     * @example
     * For google, this is `https://oauth2.googleapis.com/token`.
     */
    tokenEndpoint: string;
    /**
     * The URL of the refresh endpoint for refreshing access tokens.
     *
     * @example
     * For google, this is `https://oauth2.googleapis.com/token`.
     */
    refreshEndpoint?: string;
    /**
     * The URL of the revoke endpoint for revoking access tokens.
     *
     * @example
     * For google, this is `https://oauth2.googleapis.com/revoke`.
     */
    revokeEndpoint: string;
    /**
     * The code challenge method used for PKCE. It can either be 'S256' or null.
     */
    codeChallengeMethod: "S256" | null;
});
type TTokenCustomData = z.ZodObject<any> | null;
/**
 * Configuration for authenticating with an access token that users must generate themselves.
 *
 * This is useful for resources that don't support OAuth, but do support API keys.
 */
type TTokenAuthConfig<TCustomData extends TTokenCustomData> = TAuthConfigBase & {
    /**
     * The URL for documentation on how to generate an access token from the provider.
     *
     * This will be displayed for users when they are configuring the action to help them get setup.
     */
    generatingTokenReferenceURL: string;
    /**
     * Optional schema for custom data to be stored in the user's account that is needed to use the action.
     *
     * @example
     * For the Canvas LTS action, we also need to collect the user's Canvas domain (e.g. `https://canvas.calpoly.edu`).
     * ```
     * z.object({
     *  canvasDomain: z.string().url().refine((url) => url.startsWith("https://"), {
     *   message: "Canvas domain must start with https://",
     *  }),
     * })
     * ```
     */
    customDataSchema: TCustomData;
    /**
     * An optional function to validate the access token and custom data upon submission.
     *
     * @example
     * For the Canvas LTS action, we need to validate the access token and custom data by making a request to the Canvas API.
     *
     * ```typescript
     * validateToken: async ({ auth }) => {
     *  const response = await fetch(`${auth.customData.canvasDomain}/api/v1/users/self`, {
     *   headers: {
     *    Authorization: `Bearer ${auth.accessToken}`,
     *   },
     *  })
     *
     *  return { isValid: response.ok }
     * }
     * ```
     */
    validateToken?: (_: {
        auth: {
            customData: TCustomData extends z.ZodObject<any> ? z.infer<TCustomData> : null;
            accessToken: string;
        };
    }) => Promise<{
        isValid: boolean;
    }>;
};
/**
 * The authentication configuration for an action.
 */
type TActionAuth = {
    type: typeof AuthType.NONE;
    config: undefined;
} | {
    type: typeof AuthType.TOKEN;
    config: TTokenAuthConfig<any>;
} | {
    type: typeof AuthType.OAUTH;
    config: TOAuthConfig;
};
/**
 * Generic type for getting the auth argument in the action input function.
 *
 * The auth argument will always have an `accessToken` property, but it may also have a `customData` property if the auth config has a custom data schema.
 */
type TAuthArg<TAuth extends TActionAuth> = TAuth["config"] extends TTokenAuthConfig<any> ? {
    customData: TAuth["config"]["customDataSchema"] extends null ? null : z.infer<TAuth["config"]["customDataSchema"]>;
    accessToken: string;
} : TAuth["config"] extends TOAuthConfig ? {
    accessToken: string;
} : never;

declare class ActionBuilderWithFunction<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TActionAuth, TSubmission extends TActionOnSubmit = undefined> {
    actionData: TActionData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TAuth, TSubmission>;
    constructor({ actionData, }: {
        actionData: TActionData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TAuth, TSubmission>;
    });
    setExampleInput(exampleInput: z.infer<TInput>): this;
    getId(): TId;
    getExampleInput(): z.TypeOf<TInput> | null;
    getNamespace(): TNamespace;
    getMetadata(): TMetadata extends z.AnyZodObject ? z.input<TMetadata> : undefined;
    getAuthConfig(): TAuth;
    getInputSchema(): TInput;
    getSubmissionSchema(): TSubmission | undefined;
    getActionFunction(): TActionFunction<TExtras, TInput, TOutput, TAuth, TSubmission>;
    getOutputSchema(): TOutput;
    getInputJSONSchema(): zod_to_json_schema.JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: zod_to_json_schema.JsonSchema7Type;
        } | undefined;
    };
    getOutputJSONSchema(): zod_to_json_schema.JsonSchema7Type & {
        $schema?: string | undefined;
        definitions?: {
            [key: string]: zod_to_json_schema.JsonSchema7Type;
        } | undefined;
    };
    getComponent(): TPassThroughComponent<TMetadata, TInput, TOutput, TSubmission>;
}

type TActionBuilderWithAuthData<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TActionAuth, TSubmission extends TActionOnSubmit = undefined> = TActionBuilderWithOutputData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TSubmission> & Pick<TActionData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TAuth, TSubmission>, "authConfig">;
declare class ActionBuilderWithAuth<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TActionAuth, TSubmission extends TActionOnSubmit = undefined> {
    actionData: TActionBuilderWithAuthData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TAuth, TSubmission>;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithAuthData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TAuth, TSubmission>;
    });
    /**
     * The function that gets called when the action is invoked. This function should return a promise that resolves to the output of the action.
     *
     * @param input The input to the action function. This is the input that the user provides when invoking the action.
     * @param auth The authentication information for the action. This is the authentication information that the user provides when invoking the action.
     *
     * @returns A promise that resolves to the output of the action.
     *
     * @example
     * The following action function is an example for a `send-email` action.
     * ```
     * async ({ input, auth }) => {
     *   const { subject, body, to } = input;
     *   const { accessToken } = auth;
     *   ...code to send email using the input...
     *   return;
     * }
     * ```
     *
     * @example
     * The following action function is an example for a `get-google-contact` action.
     * ```
     * async ({ input, auth }) => {
     *   const contact = ...code to get contact...
     *   return contact;
     * }
     * ```
     */
    setActionFunction(actionFunction: TActionFunction<TExtras, TInput, TOutput, TAuth, TSubmission>): ActionBuilderWithFunction<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TAuth, TSubmission>;
}

declare class ActionBuilderWithOAuthType<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput, TOutput extends TActionOutput, TSubmission extends TActionOnSubmit = undefined> {
    actionData: TActionBuilderWithOutputData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TSubmission>;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithOutputData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TSubmission>;
    });
    setOAuthData: (data: TOAuthConfig) => ActionBuilderWithAuth<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, {
        type: "OAuth";
        config: TOAuthConfig;
    }, TSubmission>;
}

declare class ActionBuilderWithTokenType<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput, TOutput extends TActionOutput, TSubmission extends TActionOnSubmit = undefined> {
    actionData: TActionBuilderWithOutputData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TSubmission>;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithOutputData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TSubmission>;
    });
    setTokenData: <T extends TTokenCustomData>(data: TTokenAuthConfig<T>) => ActionBuilderWithAuth<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, {
        type: "Token";
        config: TTokenAuthConfig<T>;
    }, TSubmission>;
}

type TNoAuth = {
    type: typeof AuthType.NONE;
    config: undefined;
};
type TActionBuilderWithOutputData<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput, TOutput extends TActionOutput, TSubmission extends TActionOnSubmit = undefined> = TActionBuilderWithInputData<TId, TNamespace, TMetadata, TExtras, TInput> & Pick<TActionData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, any, TSubmission>, "outputSchema" | "component" | "submissionSchema">;
declare class ActionBuilderWithOutput<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput, TOutput extends TActionOutput, TSubmission extends TActionOnSubmit = undefined> {
    actionData: TActionBuilderWithOutputData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TSubmission>;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithOutputData<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TSubmission>;
    });
    /**
     * The authentication type for the action. This is used to generate the authentication page for the action and get users authenticated.
     */
    setAuthType(type: typeof AuthType.NONE): ActionBuilderWithAuth<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TNoAuth, TSubmission>;
    setAuthType(type: typeof AuthType.TOKEN): ActionBuilderWithTokenType<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TSubmission>;
    setAuthType(type: typeof AuthType.OAUTH): ActionBuilderWithOAuthType<TId, TNamespace, TMetadata, TExtras, TInput, TOutput, TSubmission>;
}

declare class ActionBuilderWithGet<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput> {
    actionData: TActionBuilderWithInputData<TId, TNamespace, TMetadata, TExtras, TInput>;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithInputData<TId, TNamespace, TMetadata, TExtras, TInput>;
    });
    /**
     * `JSON Data` that adheres to a specific Zod schema. This is typically the output for actions that `GET data`. If this is the case, you should specify the Zod schema for the output.
     */
    setOutputSchema(output: z.ZodObject<any>): ActionBuilderWithOutput<TId, TNamespace, TMetadata, TExtras, TInput, z.ZodObject<any, z.UnknownKeysParam, z.ZodTypeAny, {
        [x: string]: any;
    }, {
        [x: string]: any;
    }>, undefined>;
}

type TActionBuilderWithInputData<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput> = Pick<TActionData<TId, TNamespace, TMetadata, TExtras, TInput, any, any, any>, "metadata" | "inputSchema" | "id" | "actionFunctionExtrasSchema" | "namespace">;
declare class ActionBuilderWithInput<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput> {
    actionData: TActionBuilderWithInputData<TId, TNamespace, TMetadata, TExtras, TInput>;
    constructor({ actionData, }: {
        actionData: TActionBuilderWithInputData<TId, TNamespace, TMetadata, TExtras, TInput>;
    });
    /**
     * Spark has two options for actions:
     *
     * 1.`JSON Data` that adheres to a specific Zod schema. This is typically the output for actions that `GET data`. If this is the case, you should specify the Zod schema for the output.
     *
     * 2.`React Component` that asks for confirmation for the action. This is typically the output for actions that `POST data`. If this is the case, you should specify a React component that asks for confirmation for the action. The output will be `void`.
     */
    setActionType(type: "GET"): ActionBuilderWithGet<TId, TNamespace, TMetadata, TExtras, TInput>;
    setActionType(type: "POST"): ActionBuilderWithPost<TId, TNamespace, TMetadata, TExtras, TInput>;
}

type TActionComponentPropsData<TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit = undefined> = {
    /**
     * The input to the action function. You can use this input to prepopulate the form so all the user has to do is click submit.
     */
    input: z.infer<TInput>;
    /**
     * Indicates whether the action function is currently running.
     */
    isLoading: boolean;
    /**
     * Indicates whether the action function has completed running and is successful.
     */
    isSuccess: boolean;
    /**
     * Indicates whether the action function has completed running and has failed.
     */
    isError: boolean;
    /**
     * The submit handler for the form. This is called when the user clicks the submit button.
     */
    onSubmit: (formValues: z.infer<TOnSubmitValues extends undefined ? TInput : TOnSubmitValues>) => Promise<void>;
};
type TActionComponentProps<TMetadata extends TActionMetadata, TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit = undefined> = {
    inputSchema: TInput;
    metadata: TMetadata extends z.AnyZodObject ? z.input<TMetadata> : undefined;
} & ({
    /**
     *  When the action is active, the user can interact with the component.
     */
    displayState: "active";
    data: TActionComponentPropsData<TInput, TOnSubmitValues>;
} | {
    /**
     * A placeholder component is used for things like the action details page, where the user can view the component but not interact with it.
     */
    displayState: "placeholder";
    data: undefined;
} | {
    /**
     * Indicates whether the `LLM is still streaming in the input to the action function`.
     * If this is the case, it is necessary to show a loading state of the form.
     * We recommend using Skeleton components instead of input fields to indicate that the form is loading.
     */
    displayState: "skeleton";
    data: undefined;
});
type TActionComponent<TMetadata extends TActionMetadata, TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit = undefined> = React.FC<TActionComponentProps<TMetadata, TInput, TOnSubmitValues>>;
type TPassThroughComponent<TMetadata extends TActionMetadata, TInput extends TActionInput, TOutput extends TActionOutput, TOnSubmitValues extends TActionOnSubmit = undefined> = TOutput extends z.ZodVoid ? TActionComponent<TMetadata, TInput, TOnSubmitValues> : null;
/**
 * For actions that do not return any data (i.e. POST actions), you need to specify a component to render in the chat after the action is invoked.
 * This component should be a form asking for confirmation to submit the input to the action function.
 */
declare class ActionBuilderWithPost<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput, TSubmission extends TActionOnSubmit = undefined> {
    actionData: TActionBuilderWithInputData<TId, TNamespace, TMetadata, TExtras, TInput> & {
        submissionSchema?: TSubmission;
    };
    constructor({ actionData, submissionSchema, }: {
        actionData: TActionBuilderWithInputData<TId, TNamespace, TMetadata, TExtras, TInput>;
        submissionSchema?: TSubmission;
    });
    setComponentSubmissionSchema<T extends TActionInput>(submissionSchema: T): ActionBuilderWithPost<TId, TNamespace, TMetadata, TExtras, TInput, T>;
    setOutputComponent(component: React.FC<TActionComponentProps<TMetadata, TInput, TSubmission>>): ActionBuilderWithOutput<TId, TNamespace, TMetadata, TExtras, TInput, z.ZodVoid, TSubmission>;
}

type TActionMetadata = z.ZodObject<any> | undefined;
type TActionFunctionExtras = z.ZodObject<any> | undefined;
type TActionInput = z.ZodObject<any>;
type TActionOutput = z.ZodObject<any> | z.ZodVoid;
type TActionOnSubmit = z.ZodObject<any> | undefined;
type TActionFunction<TExtras extends TActionFunctionExtras, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TActionAuth, TSubmission extends TActionOnSubmit = undefined> = (_: {
    input: z.infer<TSubmission extends undefined ? TInput : TSubmission>;
    auth: TAuthArg<TAuth>;
    extras: TExtras extends z.AnyZodObject ? z.infer<TExtras> : undefined;
}) => Promise<z.infer<TOutput>>;
type TActionData<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, TInput extends TActionInput, TOutput extends TActionOutput, TAuth extends TActionAuth, TSubmission extends TActionOnSubmit = undefined> = {
    id: TId;
    namespace: TNamespace;
    metadata: TMetadata extends z.AnyZodObject ? z.input<TMetadata> : undefined;
    actionFunctionExtrasSchema: TExtras;
    inputSchema: TInput;
    submissionSchema?: TSubmission;
    outputSchema: TOutput;
    authConfig: TAuth;
    actionFunction: TActionFunction<TExtras, TInput, TOutput, TAuth, TSubmission>;
    component: TPassThroughComponent<TMetadata, TInput, TOutput, TSubmission>;
    exampleInput: z.infer<TInput> | null;
};

declare class ActionBuilder<TId extends string, TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras> {
    actionData: {
        metadata: TMetadata extends Zod.AnyZodObject ? Zod.input<TMetadata> : undefined;
        actionFunctionExtrasSchema: TExtras;
        id: TId;
        namespace: TNamespace;
    };
    constructor(input: {
        namespace: TNamespace;
        metadata: TMetadata extends Zod.AnyZodObject ? Zod.input<TMetadata> : undefined;
        actionFunctionExtrasSchema: TExtras;
        id: TId;
    });
    /**
     * The Zod input schema for the action. This is used to validate the arguments passed to the action.
     *
     * You must include a `.describe()` call on each field to provide a description of the input. This description will be used to generate the prompts and details page for your action.
     *
     * @example
     * The following input schema is an example for a `send-email` action.
     * ```
     * z.object({
     *  subject: z.string().desribe("The subject of the email"),
     *  body: z.string().describe("The body of the email"),
     *  to: z.array(z.object({
     *   email: z.string().describe("The email of the recipient"),
     * })).describe("The recipients of the email"),
     * ```
     */
    setInputSchema<T extends TActionInput>(input: T): ActionBuilderWithInput<TId, TNamespace, TMetadata, TExtras, T>;
}

type TActionsRegistry<TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras, T extends Array<ActionBuilderWithFunction<any, TNamespace, TMetadata, TExtras, any, any, any, any>>> = T extends [infer A, ...infer R] ? A extends ActionBuilderWithFunction<infer Id, TNamespace, TMetadata, TExtras, any, any, any, any> ? R extends ActionBuilderWithFunction<any, TNamespace, TMetadata, TExtras, any, any, any, any>[] ? Readonly<{
    [K in Id]: A;
}> & TActionsRegistry<TNamespace, TMetadata, TExtras, R> : never : never : Readonly<{}>;
/**
 * Filter the action registry by the auth type.
 *
 * @param registry The registry of actions.
 * @param authType The auth type to filter by.
 *
 * @returns The filtered registry of actions.
 */
declare const filterActionRegistryByAuthType: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any, any, any, any, any, any, any, undefined>;
}>, U extends TAuthType>(registry: T, authType: U) => { [K in keyof T as ReturnType<T[K]["getAuthConfig"]>["type"] extends U ? K : never]: T[K]; };
/**
 * Given an action registry, generate the tools for the LLM.
 *
 * @param registry The registry of actions.
 * @param toolId The id(s) of the action(s) to generate tools for. Can be a single id or an array of ids.
 *
 * @returns The tools for the LLM. In the format of `OpenAI.Chat.Completions.ChatCompletionTool[]`. The name of the tool is the ID of the action.
 *
 * @example
 */
declare const generateLLMTools: <const T extends Readonly<{
    [key: string]: ActionBuilderWithFunction<any, any, any, any, any, any, any, undefined>;
}>, U extends keyof T | [keyof T, ...(keyof T)[]]>(registry: T, toolId: U) => OpenAI.Chat.Completions.ChatCompletionTool[];
type TCreateActionsRegistryFunction<TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras> = <const T extends Array<ActionBuilderWithFunction<string, TNamespace, TMetadata, TExtras, any, any, any, any>>>(registry: T) => TActionsRegistry<TNamespace, TMetadata, TExtras, T>;
type TCreateActionFunction<TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras> = <TId extends string>(input: {
    id: TId;
} & (TMetadata extends z.AnyZodObject ? {
    metadata: z.input<TMetadata>;
} : {})) => ActionBuilder<TId, TNamespace, TMetadata, TExtras>;
/**
 * Generate the functions to create an action registry.
 *
 * @param args The arguments to generate the functions.
 * @returns The functions to create an action registry.
 *
 * @example
 * ```
 * const { createADEActionsRegistry, createADEAction } = generateActionRegistryFunctions({
 *   namespace: "ADE",
 *   metadataSchema: z.object({
 *     title: z.string(),
 *     description: z.string(),
 *   }),
 *   actionFunctionExtras: z.object({
 *     userData: z.object({
 *       email: z.string().email(),
 *       name: z.string(),
 *     })
 *   }),
 * })
 * ```
 */
declare const generateActionRegistryFunctions: <TNamespace extends string, TMetadata extends TActionMetadata, TExtras extends TActionFunctionExtras>(args: {
    /**
     * The namespace for the actions registry. Useful if you have multiple action registries.
     *
     * @example "ADE"
     */
    namespace: TNamespace;
    /**
     * The schema for the metadata of the action. This is used to validate the metadata passed to the action.
     *
     * If you don't have metadata, you can pass `undefined`.
     *
     * @example
     * ```typescript
     * z.object({
     *   title: z.string(),
     *   description: z.string(),
     * })
     * ```
     */
    metadataSchema: TMetadata;
    /**
     * Sometimes you may want to pass extra data to each action function (on top of input and auth). This is useful for things like user data, local time zone, etc.
     *
     * If you don't have any extra data, you can pass `undefined`.
     *
     * @example
     * ```typescript
     * z.object({
     *  userData: z.object({
     *   email: z.string().email(),
     *   name: z.string(),
     *  })
     * })
     * ```
     */
    actionFunctionExtras: TExtras;
}) => Readonly<{ [K in `create${TNamespace}ActionsRegistry`]: TCreateActionsRegistryFunction<TNamespace, TMetadata, TExtras>; } & { [K_1 in `create${TNamespace}Action`]: TCreateActionFunction<TNamespace, TMetadata, TExtras>; }>;
declare const createActionsRegistry: TCreateActionsRegistryFunction<"", undefined, undefined>;
declare const createAction: TCreateActionFunction<"", undefined, undefined>;
type inferActionComponent<T extends TCreateActionFunction<any, any, any>, TInput extends TActionInput, TOnSubmitValues extends TActionOnSubmit = undefined> = T extends TCreateActionFunction<any, infer TMetadata, any> ? TActionComponent<TMetadata, TInput, TOnSubmitValues> : never;

export { AuthType, type ValuesOf, createAction, createActionsRegistry, filterActionRegistryByAuthType, generateActionRegistryFunctions, generateLLMTools, type inferActionComponent };
